import time
from typing import Optional


def timeit(func, min_time=0):
    def wrapper(*args, **kwargs):
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        elapsed_time = end_time - start_time
        if elapsed_time >= min_time:
            print(f"Function '{func.__name__}' took {end_time - start_time} seconds to execute.")
        return result

    return wrapper


class TimeIt:
    start_time:float
    end_time:float
    execution_time:float
    round:Optional[int]

    def __init__(self, round: Optional[int] = None):
        self.round = round

    def __enter__(self):
        self.start_time = time.time()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.end_time = time.time()
        self.execution_time = self.end_time - self.start_time
        if self.round is not None:
            print(f"Execution time: {round(self.execution_time, self.round)} seconds")
        else:
            print(f"Execution time: {self.execution_time} seconds")



from typing import List, Union, TypeVar, Callable

T = TypeVar('T')


def flatten(arr: List[Union[T, List[T]]]) -> List[T]:
    """
        Flattens a nested list into a single-level list.

        Args:
        arr (List[Union[T, List[T]]]): The nested list to be flattened.

        Returns:
        List[T]: The flattened list.
    """
    flat_list = []
    for item in arr:
        if isinstance(item, list):
            flat_list.extend(flatten(item))
        else:
            flat_list.append(item)
    return flat_list


def merge_into(arr1: List[T], arr2: List[T], arr3: List[T]) -> List[T]:
    for i in arr2:
        if i not in arr1 and i not in arr3:
            arr1.append(i)
    return arr1


def not_equal(arr1: List[T], arr2: List[T]) -> bool:
    if len(arr1) != len(arr2):
        return True
    for i, j in zip(arr1, arr2):
        if i != j:
            return True
    return False


def transfer_to_tuple(*args: List[T]) -> List[tuple]:
    if not all(isinstance(arg, (list, tuple)) for arg in args):
        raise ValueError("All arguments must be lists or tuples.")
    return [tuple(elements) for elements in zip(*args)]


if __name__ == '__main__':
    print(transfer_to_tuple([1, 2, 3], [4, 5, 6]))



import functools
import os
import sys
import inspect


def setup_paths(verbose=True):
    caller_file = inspect.stack()[-1].filename
    caller_dir = os.path.dirname(os.path.abspath(caller_file))
    project_root = os.path.abspath(os.path.join(caller_dir, '..'))
    if project_root not in sys.path:
        sys.path.insert(0, project_root)

    if verbose:
        print(caller_file)


def log_call_position(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        stack = inspect.stack()

        calling_frame = stack[1]
        filename = calling_frame.filename
        lineno = calling_frame.lineno
        sys.stdout.write(f"函数 {func.__name__} 被调用于: 文件:行号\n")
        sys.stdout.write(f"{filename}:{lineno}\n")
        return func(*args, **kwargs)

    return wrapper


def get_call_position(func):
    try:
        source_lines, start_line = inspect.getsourcelines(func)
        file_name = inspect.getfile(func)
        return f'{file_name}:{start_line}'
    except Exception as e:
        return None, None


def find_print():
    import builtins
    original_print = builtins.print
    builtins.print = log_call_position(original_print)


def find_function(func, local=True):
    # module = inspect.getmodule(func)
    stack = inspect.stack()
    calling_frame = stack[1]
    calling_module = inspect.getmodule(calling_frame[0])
    if calling_module is not None:
        func_name = func.__name__
        decorated_func = log_call_position(func)
        setattr(calling_module, func_name, decorated_func)
        if not local:
            module = inspect.getmodule(func)
            setattr(module, func_name, decorated_func)
        # print('has settings', calling_module, func_name, decorated_func)
    else:
        raise ValueError("函数不属于任何模块")
    return log_call_position(func)



import argparse
import os
import sys


def get_hyperparameters():
    parser = argparse.ArgumentParser(description="Set hyperparameters for start server.")
    parser.add_argument("language", type=str, choices=['en', 'ch'], nargs='?', default='en',
                        help="Set the language: 'ch' for Chinese, 'en' for English")
    parser.add_argument("--config", type=str,
                        help="Set the configuration file")
    parser.add_argument("--h", action="store_true",
                        help="Print hyperparameter options.")

    args = parser.parse_args()
    if args.h:
        print("Usage: setup_paths.py [language]\n")
        print("Positional arguments:")
        print("  language    Set the language: 'ch' for Chinese, 'en' for English")
        exit()
    return vars(args)


def main():
    from pathlib import Path
    from .file_config import find_top_level_package
    parameters_dict = get_hyperparameters()
    current_dir = find_top_level_package((os.getcwd()))
    current_dir = Path(current_dir).resolve()
    print(current_dir)
    sys.path.insert(0, str(current_dir))
    file_name = 'quick_start'
    package = os.path.basename(current_dir)
    from .custom_importer import import_module
    print(package)
    module = import_module(file_name, package)

    module.main(parameters_dict)



import inspect
import ast
from .setup_paths import get_call_position
from .performer_helper import timeit
import types


def get_called_functions(func):
    if not isinstance(func, (types.FunctionType, types.MethodType)):
        print(f"Warning: {func} is not a function or method")
        return set()

    try:
        source = inspect.getsource(func)
    except OSError as e:
        print(f"Warning: Could not get source for {func.__name__}: {e}")
        return set()
    except IndentationError as e:
        print(f"Warning: Indentation error in source of {func.__name__}: {e}")
        return set()

    try:
        tree = ast.parse(source)
    except SyntaxError as e:
        print(f"Warning: Syntax error in source of {func.__name__}: {e}")
        return set()

    called_functions = set()

    class FunctionCallVisitor(ast.NodeVisitor):
        def visit_Call(self, node):
            if isinstance(node.func, ast.Name):
                if not node.func.id.startswith('_'):
                    called_functions.add(node.func.id)
            elif isinstance(node.func, ast.Attribute):
                if isinstance(node.func.value, ast.Name):
                    if not node.func.attr.startswith('_'):
                        called_functions.add(f"{node.func.value.id}.{node.func.attr}")
                elif isinstance(node.func.value, ast.Attribute):
                    if not node.func.attr.startswith('_'):
                        called_functions.add(f"{node.func.value.attr}.{node.func.attr}")
            self.generic_visit(node)

    FunctionCallVisitor().visit(tree)
    return called_functions


def find_function_in_modules(func_name, current_module):
    # 如果函数名包含属性（如对象方法）
    if '.' in func_name:
        obj_name, method_name = func_name.split('.')
        try:
            obj = getattr(current_module, obj_name)
            if hasattr(obj, method_name):
                func = getattr(obj, method_name)
                if callable(func):
                    return func
        except AttributeError:
            pass
    else:
        try:
            func = getattr(current_module, func_name)
            if callable(func):
                return func
        except AttributeError:
            pass
    return None


def find_all_called_functions(func, location=False, detect_obj=False):
    all_called_functions = []
    to_process = [(func, None, get_call_position(func))] if location else [(func, None)]

    while to_process:
        res = to_process.pop(0)
        if len(res) == 2:
            current_func, parent_func = res
        else:
            current_func, parent_func, _ = res
        if current_func in [f[0] for f in all_called_functions]:
            continue

        if location:
            all_called_functions.append((current_func, parent_func, get_call_position(current_func)))
        else:
            all_called_functions.append((current_func, parent_func))

        try:
            called_functions = get_called_functions(current_func)
        except (OSError, IndentationError, SyntaxError):
            continue

        current_module = inspect.getmodule(current_func)

        for func_name in called_functions:
            func_obj = find_function_in_modules(func_name, current_module)
            if detect_obj and func_obj is None:
                for obj_name, obj in inspect.getmembers(current_module, inspect.isclass):
                    if hasattr(obj, func_name):
                        func_obj = getattr(obj, func_name)
                        break
            if location and func_obj:
                to_process.append((func_obj, current_func, get_call_position(func_obj)))
            elif func_obj:
                to_process.append((func_obj, current_func))

    return all_called_functions


def auto_timeit(all_called_functions, min_time=1E-4):
    decorated_funcs = {}
    for item in all_called_functions:
        func = item[0]
        module = inspect.getmodule(func)
        if module:
            func_name = func.__name__
            if func_name not in decorated_funcs:
                decorated_funcs[func_name] = timeit(func, min_time=min_time)
            setattr(module, func_name, decorated_funcs[func_name])



import os


def create_character_file(character_name, location=None):
    filename = character_name + '.py' if location is None else os.path.join(location, character_name + '.py')
    if os.path.exists(filename):
        print("File already exists")
        return
    with open(filename, "w") as file:
        file.write(f"""from CharacterMeta import Character
        
        
class {character_name}(Character):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)""")

    print(f"Created {filename}")
    dirtory = os.path.dirname(filename)
    init_file = os.path.join(dirtory, '__init__.py')
    if os.path.exists(init_file):
        with open(init_file, 'r') as file:
            lines = file.readlines()

        with open(init_file, 'w') as file:
            for line in lines:
                if line.strip().startswith('from .'):
                    file.write(line)
                elif line.strip() == '__all__ = [':
                    file.write(line)
                    file.write(f"    '{character_name}',\n")
                else:
                    file.write(line)
            file.write(f"\nfrom .{character_name} import {character_name}\n")
            file.write(f"\n__all__.append('{character_name}')\n")

    else:
        with open(init_file, 'w') as file:
            file.write(f"from .{character_name} import {character_name}\n")
            file.write(f"__all__ = ['{character_name}']\n")





import requests
import os
from tqdm import tqdm
from bs4 import BeautifulSoup
import re

def download(url: str, save_dir: str, file_name: str):
    response = requests.get(url)
    os.makedirs(f'{save_dir}', exist_ok=True)
    if response.status_code == 200:
        with open(os.path.join(f'{save_dir}', f"{file_name}"), 'wb') as f:
            f.write(response.content)
    else:
        print(f"Failed to download image for {file_name}. Status code: {response.status_code}")


def to_webp(name: str) -> str:
    return f'{name}.webp'


def get_filename(name: str, extension: str) -> str:
    return f'{name}.{extension}'


def download_all(url: list, save_dir: str, file_name: list, extension: str = None, timeit: bool = False) -> None:
    if timeit:
        from performer_helper import TimeIt
        with TimeIt():
            download_all(url=url, save_dir=save_dir, file_name=file_name, extension=extension, timeit=False)
    else:
        os.makedirs(f'{save_dir}', exist_ok=True)
        if len(url) != len(file_name):
            print(f"The length of url and file_name should be the same.")
        for url, name in tqdm(zip(url, file_name), total=min(len(url), len(file_name))):
            response = requests.get(url)
            if response.status_code == 200:
                if extension is not None:
                    file_name = get_filename(name, extension)
                if os.path.isfile(os.path.join(f'{save_dir}', f"{file_name}")):
                    print(f"{file_name} already exists. Skipping...")
                    continue
                with open(os.path.join(f'{save_dir}', f"{file_name}"), 'wb') as f:
                    f.write(response.content)
            else:
                print(f"Failed to download image for {file_name}. Status code: {response.status_code}")
        print(f"All images have been downloaded to {save_dir}")


def fetch_html(url):
    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
    }
    response = requests.get(url, headers=headers)
    print(f"Request URL: {response.url}")
    print(f"Response Status Code: {response.status_code}")
    print(f"Response Headers: {response.headers}")
    # response.encoding = 'utf-8'

    return response.text


def kuaishou_download(url: str, save_dir: str, file_name: str):
    html_content = fetch_html(url)
    soup = BeautifulSoup(html_content, 'html.parser')
    if html_content.strip() == "":
        print("Fetched HTML content is empty.")
    video_element = soup.find_all('script')
    element = str(video_element[1])
    print(type(element), element)
    mp4_urls = re.findall(r'https:\\/\\/[^"]+\.mp4', element)

    # 处理反斜杠
    # mp4_urls = [url.replace('\\/', '/') for url in mp4_urls]

    for url in mp4_urls:
        print(url)


import os
import subprocess
import argparse
import sys


def get_paramsdict():
    parser = argparse.ArgumentParser(description='Process some text.')
    parser.add_argument("filename", type=str, help="Given a filename")
    parser.add_argument("args", nargs='*', help="Positional arguments")
    parser.add_argument('--kwargs', nargs='*', help='Keyword arguments in the form key=value', default={})

    args = parser.parse_args()

    kwargs = {}
    for kwarg in args.kwargs:
        key, value = kwarg.split('=')
        kwargs[key] = value

    return vars(args)


def run_python(filename, *args, **kwargs):
    if not filename.endswith('.py'):
        filename += '.py'
    abs_filename = os.path.abspath(filename)
    if not os.path.exists(abs_filename):
        print(f"File {abs_filename} does not exist")
        from .file_processing import find_file
        files = find_file(os.getcwd(), filename, False)
        if len(files) == 1:
            abs_filename = os.path.abspath(files[0])
            print(f"Found {abs_filename}")
        else:
            print(f"Found {len(files)} files matching {filename}")
            print(files)
            return
    #     command = [sys.executable, "-c", f"""
    # from my_import.setup_paths import setup_paths
    # setup_paths(verbose=False)
    # with open(r'{abs_filename}') as f:
    #     code = compile(f.read(), r'{abs_filename}', 'exec')
    #     exec(code, {{'__name__': '__main__'}})
    # """]
    # with open(abs_filename, 'r') as file:
    #     code = file.read()

    command = [sys.executable, "-c", f"""
from my_import.setup_paths import setup_paths
setup_paths(verbose=False)
import runpy
runpy.run_path(r'{abs_filename}', run_name='__main__')
"""]

    for arg in args:
        command.append(str(arg))

    for key, value in kwargs.items():
        command.append(f"--{key}")
        command.append(str(value))

    try:
        result = subprocess.run(command, capture_output=True, text=True, check=True)
        print("Output:")
        print(result.stdout)
        print("Errors:")
        print(result.stderr)
    except subprocess.CalledProcessError as e:
        print(f"Error running {filename}: {e}")
        print("Output:")
        print(e.stdout)
        print("Errors:")
        print(e.stderr)


def main(params=None):
    if params is None:
        params = get_paramsdict()
    run_python(params.get('filename'), *params.get('args'), **params.get('kwargs'))


if __name__ == '__main__':
    main(get_paramsdict())
